// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QWidget>

namespace QmlJSTools {

class FormatterSelectionWidget;
class QmlCodeStyleWidgetBase;

QmlCodeStyleWidgetBase *createQmlFormatSettingsWidget(QWidget *parent,
                                                      FormatterSelectionWidget *selection);

} // namespace QmlJSTools
