// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace ClangCodeModel {
namespace Constants {

const char GENERATE_COMPILATION_DB[]  = "ClangCodeModel.GenerateCompilationDB";
const char CLANG_ERROR[] = "Clang.Error";
const char CLANG_WARNING[] = "Clang.Warning";

const char TASK_CATEGORY_DIAGNOSTICS[] = "ClangCodeModel";

} // namespace Constants
} // namespace ClangCodeModel
