// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QtGlobal>

namespace DiffEditor {
namespace Constants {

const char DIFF_EDITOR_PLUGIN[] = "DiffEditorPlugin";

const char DIFF_EDITOR_ID[] = "Diff Editor";
const char DIFF_EDITOR_MIMETYPE[] = "text/x-patch";
const char C_DIFF_EDITOR_DESCRIPTION[] = "DiffEditor.Description";
const char SIDE_BY_SIDE_VIEW_ID[] = "DiffEditor.SideBySide";
const char UNIFIED_VIEW_ID[] = "DiffEditor.Unified";
const char SELECT_ENCODING[] = "DiffEditor.SelectEncoding";

const char G_TOOLS_DIFF[] = "QtCreator.Group.Tools.Diff";

} // namespace Constants
} // namespace DiffEditor
