// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QString>

namespace QmlDesigner {

const QString StartRewriterAmend = QStringLiteral("__start rewriter amend__");
const QString EndRewriterAmend = QStringLiteral("__end rewriter amend__");
const QString StartRewriterApply = QStringLiteral("start rewriter apply__");
const QString EndRewriterApply = QStringLiteral("__end rewriter apply__");
const QString UpdateItemlibrary = QStringLiteral("__update itemlibrary__");
const QString AddConnectionNotification = QStringLiteral("__add connection__");
const QString EditConnectionNotification = QStringLiteral("edit connection__");
} // namespace QmlDesigner
