// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Utils {

namespace Constants {

const char BEAUTIFIER_SETTINGS_GROUP[] = "Beautifier";
const char BEAUTIFIER_GENERAL_GROUP[] = "General";
const char BEAUTIFIER_AUTO_FORMAT_ON_SAVE[] = "autoFormatOnSave";

} // namespace Constants
} // namespace Utils
